#include "flip.h"

/*
#define DEBUG
*/

/*
  1. determine your drivername => [your driver name]
  2. save this file as driver_[your driver name].c
*/
#define DRIVER_NAME "line"

static int _timeout = 0;

typedef enum {
  ODD,
  EVEN
} LINE_FLAG;

static SDL_Surface * _screen = NULL;

DECLARE_PROPERTY(int,add,1<<3);
DECLARE_PROPERTY(int,width,10);

PROPERTY_VECTOR_BEGIN
  DECLARE_VECTOR(add,"move add (8)"),
  DECLARE_VECTOR(width,"line width(10)"),
PROPERTY_VECTOR_END

#define _screen_width	(_screen->w)
#define _screen_height	(_screen->h)
#define _max_count	((int)(_screen_height / add))
#define _line_num	((int)(_screen_width / width))

static SDL_Surface * line_surface_new (const SDL_Surface * const_orig,
				       LINE_FLAG flag);
static void line_surface_delete (SDL_Surface * delete);
static void line_get_property ();

/* these functions should be static */
static int timeout(){ return _timeout; }
static void flip(SDL_Surface* screen, SDL_Surface* bmp)
{
  
  /* insert your majestic flip code here */
  timebase_context t;
  int count;
  SDL_Surface * odd;
  SDL_Surface * even;
  SDL_Rect dr_odd = {0, screen->h, screen->w, 0};
  SDL_Rect dr_even = {0, screen->h, screen->w, screen->h};

  _screen = screen;
  line_get_property ();
  
  odd = line_surface_new ((const SDL_Surface *)bmp, ODD);
  even = line_surface_new ((const SDL_Surface *)bmp, EVEN);
  if (odd == NULL || even == NULL) return;
  
  timebase_init(&t);
  count = 0;
  while(count<=_max_count)
    {
      SDL_BlitSurface(odd, &dr_odd, screen, NULL);
      SDL_BlitSurface(even, NULL, screen, &dr_even);

	
      if(timebase_elapse(&t, 30 * count) > 0)
	{
//	  SDL_UpdateRect(screen, 0, 0, 0, 0);
	}
      else
	{
	  _timeout++;
	}
      dr_odd.y -= add;
      dr_odd.h += (Uint32)add;
      dr_even.y -= add;
      count++;
    }
//  SDL_UpdateRect(screen, 0, 0, 0, 0);

  line_surface_delete (odd);
  line_surface_delete (even);

  return;
}

#if 0
static char* usage()
{
  /* insert description code here */
  return DRIVER_NAME ": line?\n"
    "  properties:\n    add=n set move add.\n    width=n set line width.\n";
}
#endif

driver_vector line_vector = {
  DRIVER_NAME,
  "line?",
  1,
  /* you could use dummy_???? (ex. dummy_timeout)
     if you don't want to implement this feature */
  flip,
  timeout, 
  property_init,
};

static SDL_Surface *
line_surface_new (const SDL_Surface * const_orig,
		  LINE_FLAG flag)
{
  int i;
  SDL_Surface * new;
  SDL_Surface * converted;
  SDL_Surface * orig;

  orig =  (SDL_Surface *)const_orig;
  
  new = SDL_CreateRGBSurface (SDL_SWSURFACE,
			      orig->w,
			      orig->h,
			      32,
			      0xff000000,
			      0x00ff0000,
			      0x0000ff00,
			      0x000000ff);

  switch (flag)
    {
    case ODD:
      i=0;
      break;
    case EVEN:
      i=1;
      break;
    }
  
  converted = SDL_DisplayFormatAlpha (new);
  if(converted == NULL)
    {
      SDL_FreeSurface (new);
      return NULL;
    }
  else
    {
      SDL_FreeSurface (new);
      new = converted;
    }

  for (; i<=_line_num; i+=2)
    {
      SDL_Rect rc = {i*width, 0,
		     width, _screen_height};
      SDL_BlitSurface (orig, &rc, new, &rc);
    }

  return new;
}

static void
line_surface_delete (SDL_Surface * delete)
{
  if(delete != NULL)
    {
      SDL_FreeSurface (delete);
    }
}

static void
line_get_property ()
{
  //  char buf[4096];

  //  snprintf (buf, sizeof (buf), "%d", width);
  //  width = atoi(flip_get_property("width", buf));
  if (width < 1)
    {
      width = 1;
    }
  else if (width > (_screen_width / 2))
    {
      width = _screen_width / 2;
    }

  //  snprintf (buf, sizeof (buf), "%d", add);
  //  add = atoi(flip_get_property("add", buf));
  if (add < 1)
    {
      add = 1;
    }
  else
    {
      if ((_screen_height % add) != 0)
	{
	  int work;
	    
	  work = (int)(_screen_height / add) + 1;
	  add = _screen_height / work;
	}
      if (add > _screen_height)
	{
	  add = _screen_height;
	}
    }

#ifdef DEBUG
  fprintf (stderr, "width : %d\n", width);
  fprintf (stderr, "add : %d\n", add);
#endif
}

/* others todo:
1. insert
        extern driver_vector [your driver name]_vector;
   to drivers.h

2. insert
        &[your driver name]_vector,
   to driver_vector* flip_vector[] in main.c

3. add '[your driver name].c' to DRIVERS in Makefile
*/

/*
  Local Variables:
  mode:c
  tab-width:8
  End:
*/
