/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: driver_roller.c,v 1.1.2.2 2004/02/08 09:43:01 zinnia Exp $
 */

#include "flip.h"
#define DRIVER_NAME "roller"

DECLARE_PROPERTY(int,window,120);
DECLARE_PROPERTY(int,speed,3);
DECLARE_PROPERTY(int,step,4);

PROPERTY_VECTOR_BEGIN
DECLARE_VECTOR(window,"fading window width (120)"),
DECLARE_VECTOR(speed, "flip in n seconds(3)"),
DECLARE_VECTOR(step, "flip every n line(4)"),
PROPERTY_VECTOR_END
;

static int _timeout = 0;
static int timeout(){
  return _timeout;
}

static void flip(SDL_Surface* screen, SDL_Surface* bmp)
{

  timebase_context t;
  double alpha;
  SDL_Rect src, dst;
  GLuint texture_from;
  GLfloat texcoord_from[4];
  GLfloat texMinX, texMinY, texMaxX, texMaxY;
  GLuint texture_to;
  GLfloat texcoord_to[4];
  GLfloat textMinX, textMinY, textMaxX, textMaxY;
  int x;

  timebase_init(&t);
  texture_from = GetTexture(texcoord_from);

  texMinX = texcoord_from[0];
  texMinY = texcoord_from[1];
  texMaxX = texcoord_from[2];
  texMaxY = texcoord_from[3];

  texture_to = SDL_GL_LoadTexture(bmp, texcoord_to);
  textMinX = texcoord_to[0];
  textMinY = texcoord_to[1];
  textMaxX = texcoord_to[2];
  textMaxY = texcoord_to[3];

#define CROSS
//#undef CROSS
  for(x = screen->w + window; x >= 0; x -= step){
	  glClear( GL_COLOR_BUFFER_BIT );
	  SDL_GL_Enter2DMode();

	  glBindTexture(GL_TEXTURE_2D, texture_to);
	  glColor4d(0.0, 0.0, 0.0, 1.0);
	  glBegin(GL_TRIANGLE_STRIP);
	  glTexCoord2f(textMinX, textMinY); glVertex2i(0,   0  );
	  glTexCoord2f(textMaxX, textMinY); glVertex2i(screen->w, 0  );
	  glTexCoord2f(textMinX, textMaxY); glVertex2i(0,   screen->h);
	  glTexCoord2f(textMaxX, textMaxY); glVertex2i(screen->w, screen->h);
	  glEnd();

#ifdef CROSS
	  glBlendFunc(GL_ONE, GL_ZERO);
	  glBegin(GL_TRIANGLE_STRIP);
	  glColor4d(0.0, 0.0, 0.0, 0.0);
	  glVertex2i(0, 0);
	  glVertex2i(x - window, 0);
	  glVertex2i(0, screen->h);
	  glVertex2i(x - window, screen->h);
	  glEnd();
#endif
	  glBlendFunc(GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA);
	  glBegin(GL_TRIANGLE_STRIP);
	  glColor4d(0.0, 0.0, 0.0, 0.0);
	  glVertex2i(x - window, 0);
	  glColor4d(0.0, 0.0, 0.0, 1.0);
	  glVertex2i(x, 0);
	  glColor4d(0.0, 0.0, 0.0, 0.0);
	  glVertex2i(x - window, screen->h);
	  glColor4d(0.0, 0.0, 0.0, 1.0);
	  glVertex2i(x, screen->h);
	  glEnd();

#ifdef CROSS
	  glBlendFunc(GL_ONE_MINUS_DST_ALPHA, GL_DST_ALPHA);
	  glBindTexture(GL_TEXTURE_2D, texture_from);
	  glColor4d(0.0, 0.0, 0.0, 1.0);
	  glBegin(GL_TRIANGLE_STRIP);
	  glTexCoord2f(texMinX, texMinY); glVertex2i(0,   0  );
	  glTexCoord2f(texMaxX, texMinY); glVertex2i(screen->w, 0  );
	  glTexCoord2f(texMinX, texMaxY); glVertex2i(0,   screen->h);
	  glTexCoord2f(texMaxX, texMaxY); glVertex2i(screen->w, screen->h);
	  glEnd();

#else
	  glBlendFunc(GL_ONE, GL_ZERO);
	  glColor4d(0.0, 0.0, 0.0, 1.0);
	  glBegin(GL_TRIANGLE_STRIP);
	  glVertex2i(0,   0  );
	  glVertex2i(x - window, 0  );
	  glVertex2i(0,   screen->h);
	  glVertex2i(x - window, screen->h);
	  glEnd();
#endif

	  SDL_GL_Leave2DMode();
	  SDL_GL_SwapBuffers( );
	  if(timebase_elapse(&t, (screen->w + window - x) * speed) >= 0){
		  doEvents();
	  } else {
		  _timeout++;
	  }
  }
}

driver_vector roller_vector = {
  DRIVER_NAME,
  "roller",
  0,
  flip,
  timeout,
  property_init, /* you need not implement this */
};


