/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: driver_warp.c,v 1.2.2.2 2004/02/08 03:24:18 zinnia Exp $
 */

#include <stdlib.h>
#include "flip.h"
#define DRIVER_NAME "warp"

DECLARE_PROPERTY(int,step,4);
DECLARE_PROPERTY(int,speed,4);

PROPERTY_VECTOR_BEGIN
DECLARE_VECTOR(step,"flip every n dots(4)"),
DECLARE_VECTOR(speed,"flip in n sec(4)"),
PROPERTY_VECTOR_END
;

static int _timeout = 0;
static int timeout(){
  return _timeout;
}

static void flip(SDL_Surface* screen, SDL_Surface* bmp)
{

  timebase_context t;
  int x, y, y1;
  SDL_Rect src, dst;
  
  fprintf(stderr, "step: %d, speed: %d\n", step, speed);
  timebase_init(&t);
  for(y = 0; y < bmp->h; y += step){
    src.x = 0; src.y = y;
    src.w = bmp->w; src.h = step;

    for(y1 = y; y1 <= bmp->h - step; y1 += step){
      dst.x = 0; dst.y = y1;
      SDL_BlitSurface(bmp, &src, screen, &dst);
    }
    if(timebase_elapse(&t, speed * 1000 * y / (bmp->h)) >= 0){
//      SDL_UpdateRect(screen, 0, 0, 0, 0);
      doEvents();
    } else {
	    _timeout++;
    }
  }
}

driver_vector warp_vector = {
  DRIVER_NAME,
  "warp effect",
  1,
  flip,
  timeout,
  property_init, /* you need not implement this */
};
