/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: main.c,v 1.3.2.5 2004/02/08 09:49:19 zinnia Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>

#include "flip.h"
#include "buildno.h"

#define DEFAULT_FILENAME1 "nuriko1.bmp"
#define DEFAULT_FILENAME2 "nurinuri.bmp"

static char* rcsid = "$Id: main.c,v 1.3.2.5 2004/02/08 09:49:19 zinnia Exp $";

driver_vector* flip_vector[] = {
  &warp_vector,
  &random_vector,
//  &tv_vector,
  &scroll_vector,
  &ripple_vector,
  &block16_vector,
  &line_vector,
  &fade_vector,
  &roller_vector,
  NULL,
};

static void waitforkeypressed(SDL_Surface* screen, int sdlonly)
{
  SDL_Event e;
  while(1){
    while(SDL_PollEvent(&e)){
      switch(e.type){
      case SDL_QUIT:
	exit(1);
	break; /* NOTREACHED */
      case SDL_KEYDOWN:
	goto keypressed;
      }
    }
    SDL_Delay(20);
    if(sdlonly) doUpdate(screen);
  }
 keypressed:;
}

static void show_usage(FILE* fp, driver_vector* me, int detail)
{
  fprintf(fp, "flip  build #" BUILD_NO "  (C) Zinnia\n"
	  "%s\n\n"
	  "usage: flip [-h] -d driver [-h] -e name=value file1.bmp file2.bmp\n\n", rcsid);
  if(me != NULL){
    fprintf(fp, "%s: %s\n  properties:\n", me->name, me->desc);
    flip_print_property_list(fp);
  } else {
    driver_vector** v;
    fprintf(fp, "available driver(s):\n");
    for(v = flip_vector; *v != NULL; v++){
      if(detail > 0){
	(*v)->property_init();
	fprintf(fp, "%s: %s\n  properties:\n", (*v)->name, (*v)->desc);
	flip_print_property_list(fp);
      } else {
	fprintf(fp, "  %s\n", (*v)->name);
      }
    }
  }
  exit(1);
}

static driver_vector* lookup_driver(char* name)
{
  driver_vector** v;
  for(v = flip_vector; *v != NULL; v++){
    if(strncasecmp((*v)->name, name, strlen((*v)->name)) == 0){
      return *v;
    }
  }
  return NULL;
}

static void list_driver()
{
  driver_vector** v;

  for(v = flip_vector; *v != NULL; v++){
    fprintf(stderr, "  %s\n", (*v)->name);
  }
}

#ifdef USE_OPENGL
static GLuint texture;
static GLfloat texcoord[4];
GLuint GetTexture(GLfloat* _texcoord)
{ 
	int i;
	for(i = 0; i < 4; i++){ _texcoord[i] = texcoord[i]; }
	return texture; 
}
#endif

void doUpdate(SDL_Surface* screen)
{
#ifdef USE_OPENGL
	static int count = 0;
	 static GLfloat texMinX, texMinY, texMaxX, texMaxY;

	glClear( GL_COLOR_BUFFER_BIT );
	glBindTexture(GL_TEXTURE_2D, texture);

	if(count == 0){
		texture = SDL_GL_LoadTexture(screen, texcoord);
		texMinX = texcoord[0];
		texMinY = texcoord[1];
		texMaxX = texcoord[2];
		texMaxY = texcoord[3];

		if(texture == 0) return;
	} else {
		glTexSubImage2D(GL_TEXTURE_2D, 
				0, 0, 0,
				screen->w, screen->h,
				GL_RGBA, GL_UNSIGNED_BYTE, screen->pixels);
	}
	SDL_GL_Enter2DMode();
	glBindTexture(GL_TEXTURE_2D, texture);
	glBegin(GL_TRIANGLE_STRIP);
	glTexCoord2f(texMinX, texMinY); glVertex2i(0,   0  );
	glTexCoord2f(texMaxX, texMinY); glVertex2i(screen->w, 0  );
	glTexCoord2f(texMinX, texMaxY); glVertex2i(0,   screen->h);
	glTexCoord2f(texMaxX, texMaxY); glVertex2i(screen->w, screen->h);
	glEnd();
	SDL_GL_Leave2DMode();

    SDL_GL_SwapBuffers( );
    count++;
#else
	SDL_UpdateRect(screen, 0, 0, 0, 0);
#endif      
}
typedef struct {
	driver_vector* me;
	SDL_Surface* screen;
	SDL_Surface* bitmap;
} localctx;
static int thread_done = 0;
static int mainthread(void* data)
{
	localctx* ctx = (localctx*)data;
	ctx->me->flip(ctx->screen, ctx->bitmap);
	thread_done = 1;
}

static void mainloop(driver_vector* me, SDL_Surface* screen, SDL_Surface* bitmap)
{
  SDL_Thread* thread;
  localctx ctx;
  ctx.screen = screen;
  ctx.bitmap = bitmap;
  ctx.me = me;
  waitforkeypressed(screen, me->sdlonly);
  if(me->sdlonly){
	  thread = SDL_CreateThread(mainthread, (void*)&ctx);
	  while(thread_done == 0){
		  doUpdate(screen);
		  doEvents();
	  }
  } else {
	  mainthread((void*)&ctx);
  }
  waitforkeypressed(screen, me->sdlonly);

}

#ifdef USE_OPENGL
static void setup_opengl( int width, int height )
{
    float ratio = (float) width / (float) height;

    /* $B%7%'!<%G%#%s%0%b%G%k$O(B Gouraud ($B$J$a$i$+(B) */
    glShadeModel( GL_SMOOTH );

    /* $BN"LL$r<h$j=|$/(B */
    glCullFace( GL_BACK );
    glFrontFace( GL_CCW );
    glEnable( GL_CULL_FACE );

    /* $B>C5n;~$N?'$r%;%C%H(B */
    glClearColor( 0, 0, 0, 0 );

    /* $B%S%e!<%]!<%H$r@_Dj(B */
    glViewport( 0, 0, width, height );

    /*
     * $B<M1F9TNs$rJQ99$7!"%S%e!<%\%j%e!<%`$K%;%C%H!#(B
     */
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );
    /*
     * $BN}=,(B:
     * $B$3$l$r(B glFrustum $B$N8F$S=P$7$KCV$-49$($h!#(B
     */
    gluPerspective( 60.0, ratio, 1.0, 1024.0 );
}
#endif

int main(int argc, char *argv[])
{
  SDL_Surface* screen;
#ifdef USE_OPENGL
  SDL_Surface* shadow;
  #define SHADOW shadow
#else
  #define SHADOW screen
#endif
  char* filename1, * filename2;
  char* arg;
  int i;
  SDL_Surface* bitmap1;
  SDL_Surface* bitmap2;
  SDL_Surface* tmp;
  SDL_Rect dst;
  driver_vector* me = NULL;

  int ch;
  int timeout;

  filename1 = filename2 = NULL;

  for(i = 1; i < argc; i++){
    if(*argv[i] == '-'){
#define NEXT_ARG() (*(argv[i] + 2) == '\0' ? argv[++i] : argv[i] + 2)
      switch(*(argv[i] + 1)){
      case 'e':
	arg = NEXT_ARG();
	if(flip_set_property_expression(arg) == 0){
	  fprintf(stderr, "warning: setting '%s' failed.\n", arg);
	}
	break;
      case 'd':
	arg = NEXT_ARG();
	if(me == NULL){
	  me = lookup_driver(arg);
	  if(me == NULL){
	    show_usage(stderr, me, 0);
	    exit(1);
	  }
	  me->property_init();
	}
	break;

      case 'h':
	show_usage(stderr, me, 1);
      default:
	show_usage(stderr, me, 0);
	}
    } else {
      if(filename1 == NULL) filename1 = argv[i];
      else if(filename2 == NULL) filename2 = argv[i];
    }
  }
  if(me == NULL){
    show_usage(stderr, me, 0);
  }
  if(filename1 == NULL) filename1 = DEFAULT_FILENAME1;
  if(filename2 == NULL) filename2 = DEFAULT_FILENAME2;

  //  flip_print_properties();
#ifdef USE_OPENGL
#define ADDITIONAL_FLAGS SDL_OPENGL
#else
#define ADDITIONAL_FLAGS SDL_SWSURFACE
#endif

  if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE) < 0 ) {
    fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
    exit(1);
  }
#ifdef USE_OPENGL
    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
#endif
  atexit(SDL_Quit);

  if ( (SHADOW=SDL_SetVideoMode(320,240,32,ADDITIONAL_FLAGS)) == NULL ) {
    fprintf(stderr, "SDL_SetVideoMode: %s\n", SDL_GetError());
    exit(2);
  }
#ifdef USE_OPENGL
  screen = SDL_CreateRGBSurface(
			SDL_SWSURFACE,
			shadow->w,shadow->h,
			32,
#if SDL_BYTEORDER == SDL_LIL_ENDIAN /* OpenGL RGBA masks */
			0x000000FF, 
			0x0000FF00, 
			0x00FF0000, 
			0xFF000000
#else
			0xFF000000,
			0x00FF0000, 
			0x0000FF00, 
			0x000000FF
#endif
		       );
#endif
  bitmap1 = SDL_LoadBMP(filename1);
  if(bitmap1 == NULL){
    fprintf(stderr, "SDL_LoadBMP: %s: %s\n", filename1, SDL_GetError());
    exit(1);
  }

  tmp = SDL_LoadBMP(filename2);
  if(tmp == NULL){
    fprintf(stderr, "SDL_LoadBMP: %s: %s\n", filename2, SDL_GetError());
    exit(1);
  }

  bitmap2 = SDL_ConvertSurface(screen, screen->format, screen->flags);
  dst.x = dst.y = 0;
  dst.w = tmp->w; dst.h = tmp->h;
  SDL_BlitSurface(tmp, NULL, bitmap2, &dst);
  SDL_FreeSurface(tmp);

  dst.x = dst.y = 0;
  dst.w = bitmap1->w; dst.h = bitmap1->h;
  SDL_BlitSurface(bitmap1, NULL, screen, &dst);
  SDL_FreeSurface(bitmap1);


#ifdef USE_OPENGL
  setup_opengl(screen->w, screen->h);
#endif

  /*if(me->autoupdate) */doUpdate(screen);
  mainloop(me, screen, bitmap2);

  //  flip_delete_property_all();

  timeout = me->timeout();
  if(timeout != -1){
    fprintf(stderr, "timeout: %d\n", timeout);
  }
  exit(0);
}
