/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: timebase.c,v 1.2.2.1 2004/02/08 03:24:18 zinnia Exp $
 */

#include "timebase.h"

void timebase_init(timebase_context* t)
{
  t->origin = SDL_GetTicks();
  t->last = SDL_GetTicks();
}

/*
  = 0   just
  < 0   needs to fasten
  > 0   needs to wait
*/

static int timebase_elapse_core(timebase_context* t, Uint32 start, Uint32 estimated)
{
  Uint32 now, elapsed;
  elapsed = SDL_GetTicks() - start;
  if(elapsed < estimated){
    SDL_Delay(estimated - elapsed);
  }

  t->last = SDL_GetTicks();
  return estimated - elapsed;

}

int timebase_elapse(timebase_context* t, Uint32 estimated)
{
	return timebase_elapse_core(t, t->origin, estimated);
}


int timebase_elapse_relative(timebase_context* t, Uint32 estimated)
{
	return timebase_elapse_core(t, t->last, estimated);
}
